<?php
/* OpenSSL Wrapper Class
 * Version 1.0.1
 * Copyright 2004-2005, Steve Blinch
 * http://code.blitzaffe.com
 *
 * This class acts as a wrapper around the OpenSSL functions provided by the
 * PHP OpenSSL extension, which may be intimidating to those who simply want
 * to use secure encryption, and are unfamiliar with OpenSSL's inner workings.
 *
 * This class focuses explicitly on data encryption/decryption; other features
 * of OpenSSL such as signing/etc. are not implemented.
 *
 * This class encapsulates functionality to do the following:
 *
 *	- Generate a private key/certificate pair
 *	- Encrypt data using a public or private key
 *	- Decrypt data using a public or private key
 *
 * NOTE: Encryption/decryption only works on strings up to around 117 characters.
 * You would normally use this to generate a secure key for a symmetric encryption
 * algorithm such as RC4.
 *
 *
 * EXAMPLE
 *
 * // OpenSSL wrapper class example
 * require_once(dirname(__FILE__).'/class_OpenSSL.php');
 * $openssl = &new OpenSSL();
 *
 * // generate keypair
 * $ssl->set_distinguished_name('CA','BC','Nanaimo','YourCo Inc','Web team','Your Name','you@example.com');
 * $ssl->generate_keypair('secretpassword');
 * echo "Private key: ".$ssl->private_key."\n";
 * echo "Certificate: ".$ssl->certificate."\n";
 * echo "CSR: ".$ssl->csr."\n";
 *
 * // now, encrypt some data with your public key, which can only be decrypted by your
 * // private key
 * $mydata = "secret data";
 * $encrypted = $ssl->public_key_encrypt($data,$ssl->certificate);
 * echo "Encrypted data: ".$encrypted."\n";
 *
 * $decrypted = $ssl->private_key_decrypt($encrypted,$ssl->private_key,'secretpassword');
 * echo "Decrypted data: ".$decrypted." (should be: ".$mydata.")\n";
 *
 *
 * // now, encrypt some data with your private key, which can only be decrypted by your
 * // public key (essentially validating your identity)
 * $mydata = "secret data";
 * $encrypted = $ssl->private_key_encrypt($data,$ssl->private_key,'secretpassword');
 * echo "Encrypted data: ".$encrypted."\n";
 *
 * $decrypted = $ssl->public_key_decrypt($encrypted,$ssl->certificate);
 * echo "Decrypted data: ".$decrypted." (should be: ".$mydata.")\n";
 *
 *
 * LICENSE
 *
 * This script is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This script is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *	
 * You should have received a copy of the GNU General Public License along
 * with this script; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
class OpenSSL {
	
	function OpenSSL() {
		$this->distinguished_name = false;
	}
	
	function capable() {
		return function_exists('openssl_pkey_new');
	}
	
	// Set the distinguished name for keypair generation.  Usage:
	//
	// $country - 2-letter abbreviation of the country name
	// $state - 2-letter abbreviation of the state/province name
	// $city - full name of city or locality
	// $organization_name - name of organization or company
	// $organization_dept - name of department or unit withint the specified organization
	// $name - name of primary contact person (where applicable) or domain name
	// $email - E-mail address for primary contact person
	function set_distinguished_name($country,$state,$city,$organization_name,$organization_dept,$name,$email) {
		$this->distinguished_name = array( 
		   "countryName" => $country, 
		   "stateOrProvinceName" => $state, 
		   "localityName" => $city, 
		   "organizationName" => $organization_name, 
		   "organizationalUnitName" => $organization_dept, 
		   "commonName" => $name, 
		   "emailAddress" => $email 
		); 
	}
	
	function generate_keypair($passphrase = NULL,$valid_days = 99999) {
		
		if (!$this->distinguished_name) {
			$this->error = "Must set distinguished name via ::set_distinguished_name()";
			return false;
		}
		
		// Generate a new private (and public) key pair 
		if (!$private_key = openssl_pkey_new()) {
			$this->error = "OpenSSL private key generation failed";
			return false;
		}
		
		// Generate a certificate signing request 
		if (!$csr = openssl_csr_new($this->distinguished_name,$private_key)) {
			$this->error = "OpenSSL certificate signing request generation failed";
			return false;
		}
		
		// You will usually want to create a self-signed certificate at this 
		// point until your CA fulfills your request. 
		// This creates a self-signed cert that is valid for 365 days 
		if (!$sscert = openssl_csr_sign($csr, null, $private_key, $valid_days)) {
			$this->error = "OpenSSL self-signed certificate generation failed";
			return false;
		}
		
		// Now you will want to preserve your private key, CSR and self-signed 
		// cert so that they can be installed into your web server, mail server 
		// or mail client (depending on the intended use of the certificate). 
		// This example shows how to get those things into variables, but you 
		// can also store them directly into files. 
		// Typically, you will send the CSR on to your CA who will then issue 
		// you with the "real" certificate. 
		if (!openssl_csr_export($csr, $csr_text)) { //and debug_zval_dump($csrout); 
			$this->error = "OpenSSL CSR export failed";
			return false;
		}
		if (!openssl_x509_export($sscert, $certificate_text)) { // and debug_zval_dump($certout); 
			$this->error = "OpenSSL x509 certificate export failed";
			return false;
		}
		if (!openssl_pkey_export($private_key, $private_key_text, $passphrase)) { // and debug_zval_dump($pkeyout); 	
			$this->error = "OpenSSL private key export failed";
			return false;
		}
		$this->private_key = $private_key_text;
		$this->certificate = $certificate_text;
		$this->csr = $csr_text;
		
		return true;
	}
	
	function public_key_encrypt($data,$certificate=false) {
		if (!$certificate) $certificate = $this->certificate;

		if (!$public = openssl_get_publickey($certificate)) {
			$this->error = "Invalid x509 certificate";
			return false;
		}
		
		if (!openssl_public_encrypt($data, $crypted, $public)) {
			$this->error = "OpenSSL public key encryption failed";
			return false;
		}
		$data = base64_encode($crypted);

		return $data;
	}

	function private_key_decrypt($data,$private_key=false,$passphrase=NULL) {
		if (!$private_key) $private_key = $this->private_key;

		if (!$private = openssl_get_privatekey($private_key,$passphrase)) {
			$this->error = "Invalid private key";
			return false;
		}

		$crypted = base64_decode($data);
		if (!openssl_private_decrypt($crypted, $decrypted, $private)) {
			$this->error = "OpenSSL private key decryption failed";
			return false;
		}

		return $decrypted;
	}
	
	
	function private_key_encrypt($data,$private_key=false,$passphrase=NULL) {
		if (!$private_key) $private_key = $this->private_key;
		
		if (!$private = openssl_get_privatekey($private_key,$passphrase)) {
			$this->error = "Invalid private key";
			return false;
		}
		
		if (!openssl_private_encrypt($data, $crypted, $private)) {
			$this->error = "OpenSSL private key encryption failed";
			return false;
		}
		$data = base64_encode($crypted);

		return $data;
	}

	function public_key_decrypt($data,$certificate=false) {
		if (!$certificate) $certificate = $this->certificate;
		if (!$public = openssl_get_publickey($certificate)) {
			$this->error = "Invalid x509 certificate";
			return false;
		}

		$crypted = base64_decode($data);
		if (!openssl_public_decrypt($crypted, $decrypted, $public)) {
			$this->error = "OpenSSL public key decryption failed";
			return false;
		}

		return $decrypted;
	}

}
?>